<?php

namespace Drupal\readmore_js\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Html;
use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'read_more_field_formatter' formatter.
 *
 * @FieldFormatter(
 *   id = "read_more_field_formatter",
 *   label = @Translation("readmore.js"),
 *   field_types = {
 *     "text",
 *     "text_long",
 *     "text_with_summary",
 *     "string",
 *     "string_long"
 *   }
 * )
 */
class ReadMoreFieldFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return array(
      'more_link' => 'Read more',
      'more_link_classes' => '',
      'close_link' => 'Close',
      'close_link_classes' => '',
    ) + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
   public function settingsForm(array $form, FormStateInterface $form_state) {
       $element = parent::settingsForm($form, $form_state);

       $element['more_link'] = array(
         '#title' => $this->t('Read more label'),
         '#type' => 'textfield',
         '#size' => 10,
         '#default_value' => $this->getSetting('more_link'),
       );

       $element['close_link'] = array(
         '#title' => $this->t('Close label'),
         '#type' => 'textfield',
         '#size' => 10,
         '#default_value' => $this->getSetting('close_link'),
       );
       $element['more_link_classes'] = array(
         '#title' => $this->t('Read more classes'),
         '#type' => 'textfield',
         '#size' => 10,
         '#default_value' => $this->getSetting('more_link_classes'),
       );

       $element['close_link_classes'] = array(
         '#title' => $this->t('Close classes'),
         '#type' => 'textfield',
         '#size' => 10,
         '#default_value' => $this->getSetting('close_link_classes'),
       );

       return $element;
   }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [
      $this->getSetting('more_link'),
      $this->getSetting('close_link')
    ];
    // Implement settings summary.

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      // Create selector for readmore.js
      $selector = 'field--readmore-js--' . $items->getName();
      // Return config.
      $elements['#attached']['drupalSettings']['readmore_js'][$items->getName()] = [
        'selector' => $selector,
        'more_link' => $this->getSetting('more_link'),
        'close_link' => $this->getSetting('close_link'),
        'more_link_classes' => $this->getSetting('more_link_classes'),
        'close_link_classes' => $this->getSetting('close_link_classes')
      ];
      // Append unique class.
      $elements['#attributes']['class'][] = $selector;
      // Append Field Value.
      $elements[$delta] = ['#markup' => $this->viewValue($item)];

    }

    $elements['#attached']['library'][] = 'readmore_js/readmore_js.library';
    $elements['#attached']['library'][] = 'readmore_js/readmore_js.selector';

    return $elements;
  }

  /**
   * Generate the output appropriate for one field item.
   *
   * @param \Drupal\Core\Field\FieldItemInterface $item
   *   One field item.
   *
   * @return string
   *   The textual output generated.
   */
  protected function viewValue(FieldItemInterface $item) {

    return $item->value;
  }

}
